import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import ch.ethz.ssh2.Connection;
import ch.ethz.ssh2.InteractiveCallback;
import ch.ethz.ssh2.KnownHosts;
import ch.ethz.ssh2.ServerHostKeyVerifier;
import ch.ethz.ssh2.Session;
import ch.ethz.ssh2.SCPClient;

/**
    AutoGrader - a program that automatically grades programming assignments.
    Copyright (C) 2006  Pete Nordquist   (nordquip at sou.edu)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License along
    with this program; if not, write to the Free Software Foundation, Inc.,
    51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
	(See the file COPYING)


	This program uses code that is Copyright (c) 2005 - 2006 Swiss Federal
	Institute of Technology (ETH Zurich),
	Department of Computer Science (http://www.inf.ethz.ch),
	Christian Plattner. All rights reserved.
	(See the file LICENSE.txt)

 * Handles all details of using SCP to move files between server and client
 * This code derived from Christian Plattner's SwingShell example:
	 Website: http://www.ganymed.ethz.ch/ssh2. 
 * See license information above.
 * @author Pete Nordquist
 * @version 071002 (PN) original
 */
public class SCP
{
	// Host information
	private final String HOSTNAME = "<your_host_name>";
	private final String USERNAME = "<user_name>";
	private final String PASSWORD = "<user_password>";
	private final String SERVERBASEPATH = "~/ag/";

	// Constants
	public static final int GET = 0;
	public static final int PUT = 1;

	// in-memory hosts database
	private KnownHosts database = new KnownHosts();

	// objects needed to do the transfer
	private Connection conn = null;
	private SCPClient scpc = null;

	/**
	 * This ServerHostKeyVerifier creates a hostKey 
	 * in the in-memory database.
	 *
	 */
	class AdvancedVerifier implements ServerHostKeyVerifier
	{
		public boolean verifyServerHostKey(String hostname,
			int port, String serverHostKeyAlgorithm,
			byte[] serverHostKey) throws Exception
		{
			/* Be really paranoid. We use a hashed hostname entry */
			String hashedHostname = KnownHosts.createHashedHostname(hostname);

			/* Add the hostkey to the in-memory database */
			database.addHostkey(new String[] { hashedHostname },
				serverHostKeyAlgorithm, serverHostKey);

			return true;
		}
	}

	/**
	 * The logic that one has to implement if "keyboard-interactive"
	 * autentication shall be supported.
	 * "keyboard-interactive" is a type of authentication -- the autograder
	 * supplies the password without any actual interaction with the user.
	 *
	 */
	class InteractiveLogic implements InteractiveCallback
	{

		// the callback may be invoked several times,
		// depending on how many questions-sets the server sends

		public String[] replyToChallenge(String name,
			String instruction,
			int numPrompts,
			String[] prompt,
			boolean[] echo) throws IOException
		{
			int promptCount = 0;
			String[] result = new String[numPrompts];

			// this method gets called even when numPrompts == 0
			for (int i = 0; i < numPrompts; i++)
			{
				// enter password for every prompt -- should be only one
				result[i] = PASSWORD;
				promptCount++;
			}

			return result;
		}
	}

	/**
	 * Open a connection.
	 */
	public SCP() throws Exception
	{

		// create connection
		conn = new Connection(HOSTNAME);

		/*
		 * CONNECT AND VERIFY SERVER HOST KEY (with callback)
		 */
		conn.connect(new AdvancedVerifier());

		/*
		 * AUTHENTICATION PHASE
		 */
		InteractiveLogic il = new InteractiveLogic();

		boolean res =
			conn.authenticateWithKeyboardInteractive(USERNAME, il);

		if (res != true)
			System.err.println("SCP: Server Authorization failed.");
		else
			scpc = new SCPClient(conn);
	}

	/**
	 * Close the connection
	 */
	public void close()
	{
		conn.close();
	}

	/**
	 * Transfers a file or directory to or from the server.
	 * Server names are relative to SERVERBASEPATH;
	 * local names are full path.
	 * @param getOrPut
	 * @param sourceName - name of file or dir to be transferred
	 * @param targetDirName - name of directory to copy it to
	 * must be a directory name.
	 * Propogates all exceptions.
	 */
	public void transfer( int getOrPut,
		String sourceName,
		String targetDirName) throws Exception
	{
		if (scpc == null || (getOrPut != GET && getOrPut != PUT))
			System.err.println(
				"SCP : Internal error.  Unable to do transfer.");
		if (getOrPut == GET)
			scpc.get(SERVERBASEPATH + sourceName, targetDirName);
		else 
			scpc.put(sourceName, SERVERBASEPATH + targetDirName);
	}
}
